/*
 *   TABFITS:   turn a table into a fits file - for BOP only
 *
 *	4-dec-98	coded, quick and dirty as usual		PJT
 *		  ** oops, still ccd, not fits yet **
 *      7-dec-98    read commented header instead               pjt
 *     17-dec-98    V2.0:  nxy= is now nx=, ny=                 pjt
 *      5-jan-98    V2.1 fixed header item parsing              pjt
 */

#include <stdinc.h>
#include <getparam.h>
#include <fitsio.h>
#include <ctype.h>

string defv[] = {
    "in=???\n		Input sig (table) file",
    "out=???\n		Output image file",
    "dcol=1\n           Column for data",
    "nx=\n              X-Size of data, or else specify NAXIS1 in header",
    "ny=\n              Y-Size of data, or else specify NAXIS2 in header",
    "nmax=100000\n      Allocation space for piped I/O",
    "VERSION=2.0\n	17-dec-98 PJT",
    NULL,
};

string usage = "convert tables into CCD/ images (soon to be FITS)";

#define MAXCOL 16
#define MAXDAT 2048

int get_key(char *line, char *key, char *value);

void nemo_main()
{
    stream outstr;
    char line[80];
    stream instr;
    double freq, freqold, data[MAXCOL];
    int nmax, nx, ny, i, j, ierr, naxis[2], nheader=0;
    real rmin, rmax;
    string name = getparam("in");
    int dcol = getiparam("dcol");
    char namex[32], namey[32], key[32], value[80];
    FITS *fitsfile = NULL;
    float rvalue, rdata[MAXDAT];

    if (dcol < 1 || dcol > MAXCOL) error("Bad dcol=%d\n",dcol);
    dcol--;
    
    nmax = file_lines(name, getiparam("nmax"));
    instr = stropen(name,"r");

    nx = naxis[0] = hasvalue("nx") ? getiparam("nx") : 0;
    ny = naxis[1] = hasvalue("ny") ? getiparam("ny") : 0;


    while (1) {             /* read commented header values */

        if (fitsfile==NULL && nx*ny > 0) {
            dprintf(0,"Creating fits primary HDU %d*%d\n",nx,ny);
            fitsfile = fitopen(getparam("out"),"new",2,naxis);
            if (fitsfile==NULL)
                error("Could not open fitsfile for writing");
        }
    
        get_line(instr, line);

        if (!get_key(line,key,value)) {
            dprintf(0,"Read %d header lines\n",nheader);
	    break;
	}
        nheader++;
        
        dprintf(0,"HEADER: %-8s = %s\n",key,value);
        if (streq(key,"NAXIS1")) {
            nx = naxis[0] = atoi(value);
        } else if (streq(key,"NAXIS2")) {
            ny = naxis[1] = atoi(value);
        } else if (strncmp(key,"CRVAL",5)==0 ||
                 strncmp(key,"CDELT",5)==0 ||        
                 strncmp(key,"CRPIX",5)==0) {
            /* write real fits header */
            rvalue = atof(value);
            fitwrhdr(fitsfile,key,rvalue);
        } else if (strncmp(key,"CTYPE",5)==0) {
            fitwrhd(fitsfile,key,value);
        } else if (*key) {
            /* write keyword verbose */
            /* fitwrhda(fitsfile,key,value); */
            fitwrhd(fitsfile,key,value);
        } else {
            fitwra(fitsfile,"        ",line);
        }
    }

    if (nx*ny <= 0 || fitsfile == NULL) {         
    	if (nx == 0 && ny == 0) {
    	    error("Need to specify at least nx= or ny=");
        } else if (nx == 0) {
            nx = nmax/ny;
            if (nmax % ny) warning("%d odd trailing lines in file",nmax%ny);
        } else if (ny == 0) {
            ny = nmax/nx;
            if (nmax % nx) warning("%d odd trailing lines in file",nmax%nx);
        } 

        if (nx*ny <= 0)
            error("[%d,%d] I give up, cannot determine size of array",nx,ny);

        naxis[0] = nx;
        naxis[1] = ny;

        dprintf(0,"Creating fits primary HDU %d*%d\n",nx,ny);
        fitsfile = fitopen(getparam("out"),"new",2,naxis);
        if (fitsfile==NULL)
                error("Could not open fitsfile for writing");
                
    }
        
    for (j=0; j<ny; j++) {                       /* loop over whole array */
        for (i=0; i<nx; i++) {
            if (i>0 || j>0) get_line(instr, line);      /* already read 1st */
#if 1
            if ((ierr=nemoinpd(line,data,MAXCOL)) < 1)
                error("(%d,%d) badly formatted line: \"%s\" err=%d\n",
                        i,j,line,ierr);
            rdata[i] = data[dcol];
#else
            data[dcol] = atof(line);
#endif
            if (i==0 && j==0) {
                rmin = rmax = data[dcol];
            } else {
                rmin = MIN(rmin, data[dcol]);
                rmax = MAX(rmax, data[dcol]);
            }
        }
        fitwrite(fitsfile,j,rdata);
    }
    dprintf(0,"MinMax in map: %g - %g\n",rmin,rmax);    
    fitclose(fitsfile);
}

int get_key(char *line, char *key, char *value)
{
    char *cp = line, *lkey=key, *lvalue=value;
    int has_equals;

    *key = 0;                               /* initialize no keyword */
    *value = 0;                             /* and no value part */

    if (cp==0) return 1;
    while (isspace(*cp))                    /* skip leading whitespace */
        cp++;
    if (cp==0) return 1;                    /* if only whitespace, done */
    
    if (*cp == '#')			    /* expect a comment for header */
	cp++;    
    else
    	return 0;                           /* if not, return bad line */
    	
    while (isspace(*cp))                    /* skip leading whitespace */
        cp++;  
    while (!isspace(*cp) && *cp != '=')     /* grab the keyword */
        *lkey++ = *cp++;
    while (isspace(*cp))
        cp++;        
    if (*cp == '=')			
        *lkey = 0;			    /* terminate keyword */
    else {
        *key = 0;			    /* no keyword */
        return 1;
    }
    while (isspace(*cp) || *cp == '=')      /* skip whitespace around '=' */
        cp++;
    while (*cp)                             /* grab the value */
        *value++ = *cp++;
    *value = 0;

    if (strlen(key) > 8) warning("key %s too long for FITS",key);

    return 1;
  
}

