/* 
 * CCDMOM: take a moment along an axis in an image
 *
 *	quick and dirty:  8-jun-95		pjt
 *      19-oct-95   very dirty axis=3           pjt
 *	12-dec-98   0.3  more implementations   pjt
 *      31-dec-98   0.4  keep reduced axis, and replace it with new value PJT
 *                      
 */


#include <stdinc.h>
#include <getparam.h>
#include <vectmath.h>
#include <filestruct.h>
#include <image.h>

string defv[] = {
        "in=???\n       Input image file",
	"out=???\n      Output image file",
	"axis=1\n       Axis to take moment along (1=x 2=y 3=z)",
	"mom=0\n	Moment to take [0,1,2]",
	"keep=f\n	Keep moment axis, and replace values",
	"VERSION=0.4\n  31-dec-98 PJT",
	NULL,
};

string usage = "moment along an axis of an image (unfinished)";



void nemo_main()
{
    stream  instr, outstr;
    int     nx, ny, nz, nx1, ny1, nz1;
    int     axis, mom;
    int     i,j,k;
    imageptr iptr=NULL, iptr1=NULL, iptr2=NULL;      /* pointer to images */
    real    tmp1, tmp2, newvalue;
    bool    Qkeep = getbparam("keep");

    instr = stropen(getparam("in"), "r");
    mom = getiparam("mom");
    axis = getiparam("axis");
    if (mom < 0 || mom > 2)  error("Illegal value mom=%d",mom);

    read_image( instr, &iptr);

    nx1 = nx = Nx(iptr);	
    ny1 = ny = Ny(iptr);
    nz1 = nz = Nz(iptr);
    if (Qkeep) {
        dprintf(0,"Keeping %d*%d*%d cube\n",nx1,ny1,nz1);
    } else {
        if (axis==1) {
            nx1 = 1;    ny1 = ny;   nz1 = nz;
        } else if (axis==2) {
            nx1 = nx;   ny1 = 1;    nz1 = nz;
        } else if (axis==3) {
            nx1 = nx;   ny1 = ny;   nz1 = 1;
        } else
            error("Invalid axis: %d (Valid: 1,2,3)",axis);
        dprintf(0,"Reducing %d*%d*%d to a %d*%d*%d cube\n",
                   nx,ny,nz, nx1,ny1,nz1);
    }

    outstr = stropen(getparam("out"), "w");

    create_cube(&iptr1,nx1,ny1,nz1);
    create_cube(&iptr2,nx1,ny1,nz1);

    if (axis==1) {                             
      for (k=0; k<nz; k++)
        for (j=0; j<ny; j++) {
            tmp1 = tmp2 = 0.0;
            for (i=0; i<nx; i++) {
                tmp1 += CubeValue(iptr,i,j,k);
                tmp2 += sqr(CubeValue(iptr,i,j,k));
            }
            tmp1 /= nx;     /* mean */
            if (mom==0) 
                newvalue = tmp1 * nx;
            else if (mom==1)
                newvalue = tmp1;
            else if (mom=2)
                newvalue = sqrt(tmp2/nx - tmp1*tmp1);
            for (i=0; i<nx1; i++)
                CubeValue(iptr1,i,j,k) = newvalue;
        }

        /* TODO: fix up Qkeep headers */

        Xmin(iptr1) = Xmin(iptr) + 0.5*(nx-1)*Dx(iptr);
        Ymin(iptr1) = Ymin(iptr);
        Zmin(iptr1) = Zmin(iptr);
        Dx(iptr1) = nx * Dx(iptr);
        Dy(iptr1) = Dy(iptr);
        Dz(iptr1) = Dz(iptr);
        
        Namex(iptr1) = Namex(iptr); /* care: we're passing a pointer */
        Namey(iptr1) = Namey(iptr);
        Namez(iptr1) = Namez(iptr);

    } else if (axis==2) {                      
      for (k=0; k<nz; k++)
        for (i=0; i<nx; i++) {
            tmp1 = tmp2 = 0.0;
            for (j=0; j<ny; j++) {
                tmp1 += CubeValue(iptr,i,j,k);
                tmp2 += sqr(CubeValue(iptr,i,j,k));
            }
            tmp1 /= ny;
            if (mom==0)
                newvalue = tmp1 * ny;
            else if (mom==1)
                newvalue = tmp1;
            else if (mom==2)
                newvalue = sqrt(tmp2/ny - tmp1*tmp1);
            for (j=0; j<ny1; j++)
                CubeValue(iptr1,i,j,k) = newvalue;
        }

        /* TODO: */

        Xmin(iptr1) = Xmin(iptr);
        Ymin(iptr1) = Ymin(iptr) + 0.5*(ny-1)*Dy(iptr);
        Zmin(iptr1) = Zmin(iptr);
        Dx(iptr1) = Dx(iptr);
        Dy(iptr1) = ny * Dy(iptr);
        Dz(iptr1) = Dz(iptr);
        
        Namex(iptr1) = Namex(iptr); /* care: we're passing a pointer */
        Namey(iptr1) = Namey(iptr);
        Namez(iptr1) = Namez(iptr);

    } else if (axis==3) {                       /* this one is tested */
    	for(j=0; j<ny; j++)
    	for(i=0; i<nx; i++) {
    	    tmp1 = tmp2 = 0.0;
    	    for(k=0; k<nz; k++) {
    	        tmp1 += CubeValue(iptr,i,j,k);
    	        tmp2 += sqr(CubeValue(iptr,i,j,k));
    	    }
    	    tmp1 /= nz;
    	    if (mom==0)
    	        newvalue = tmp1 * nz;
    	    else if (mom==1)
    	        newvalue = tmp1;
    	    else if (mom==2)
      	        newvalue = sqrt(tmp2/nz - tmp1*tmp1);
            for (k=0; k<nz1; k++)
                CubeValue(iptr1,i,j,k) = newvalue;
    	}

        /* TODO: */

        Xmin(iptr1) = Xmin(iptr);
        Ymin(iptr1) = Ymin(iptr);
        Zmin(iptr1) = Zmin(iptr) + 0.5*(nz-1)*Dz(iptr);
        Dx(iptr1) = Dx(iptr);
        Dy(iptr1) = Dy(iptr);
        Dz(iptr1) = nz * Dz(iptr);
        
        Namex(iptr1) = Namex(iptr); /* care: we're passing a pointer */
        Namey(iptr1) = Namey(iptr);
        Namez(iptr1) = Namez(iptr);
        
    } else
        error("Cannot do axis %d",axis);

    write_image(outstr, iptr1);
}
